import { Type } from '@angular/core';
import { SafeAny } from '@ngify/types';
import { SchemaConfig } from '../interfaces';
import { SchemaPatchFn } from '../patcher';
import { AbstractSchema, AlertComponentSchema, ButtonComponentSchema, ButtonGroupComponentSchema, CascaderControlSchema, CheckboxControlSchema, CheckboxGroupControlSchema, DatePickerControlSchema, DateRangePickerControlSchema, FormArraySchema, FormGroupSchema, HeadingComponentSchema, InputControlSchema, InputGroupComponentSchema, NumberGroupComponentSchema, NumberInputControlSchema, RadioGroupControlSchema, RateControlSchema, SelectControlSchema, SliderControlSchema, StepComponentSchema, StepsComponentSchema, TabComponentSchema, TabsComponentSchema, TextComponentSchema, TextareaControlSchema, TimePickerControlSchema, ToggleControlSchema, TreeSelectControlSchema } from '../schemas';
import { AbstractWidget } from '../widgets';
import { FluentFormFeature, FluentFormFeatureKind } from './interface';
export interface FluentFormWidgetFeature<S extends AbstractSchema> extends SchemaConfig<S> {
    kind: string;
    widget?: Type<AbstractWidget<unknown>>;
    /** 修补图示，标准化图示时调用 */
    patch?: SchemaPatchFn<S>;
}
export declare function withWidgets(features: (FluentFormWidgetFeature<SafeAny> | FluentFormWidgetFeature<SafeAny>[])[]): FluentFormFeature<FluentFormFeatureKind.Widget>;
export declare function withAllWidgets(): FluentFormFeature<FluentFormFeatureKind.Widget>;
export declare function useInputWidget(): FluentFormWidgetFeature<InputControlSchema>;
export declare function useTextareaWidget(): FluentFormWidgetFeature<TextareaControlSchema>;
export declare function useNumberWidget(): FluentFormWidgetFeature<NumberInputControlSchema>;
export declare function useDateWidget(): FluentFormWidgetFeature<DatePickerControlSchema>;
export declare function useDateRangeWidget(): FluentFormWidgetFeature<DateRangePickerControlSchema>;
export declare function useTimeWidget(): FluentFormWidgetFeature<TimePickerControlSchema>;
export declare function useToggleWidget(): FluentFormWidgetFeature<ToggleControlSchema>;
export declare function useSelectWidget(): FluentFormWidgetFeature<SelectControlSchema>;
export declare function useCascaderWidget(): FluentFormWidgetFeature<CascaderControlSchema>;
export declare function useTreeSelectWidget(): FluentFormWidgetFeature<TreeSelectControlSchema>;
export declare function useSliderWidget(): FluentFormWidgetFeature<SliderControlSchema>;
export declare function useRadioGroupWidget(): FluentFormWidgetFeature<RadioGroupControlSchema>;
export declare function useCheckboxWidget(): FluentFormWidgetFeature<CheckboxControlSchema>;
export declare function useCheckboxGroupWidget(): FluentFormWidgetFeature<CheckboxGroupControlSchema>;
export declare function useRateWidget(): FluentFormWidgetFeature<RateControlSchema>;
export declare function useTextWidget(): FluentFormWidgetFeature<TextComponentSchema>;
export declare function useHeadingWidget(): FluentFormWidgetFeature<HeadingComponentSchema>;
export declare function useInputGroupWidget(): FluentFormWidgetFeature<InputGroupComponentSchema>;
export declare function useNumberGroupWidget(): FluentFormWidgetFeature<NumberGroupComponentSchema>;
export declare function useAlertWidget(): FluentFormWidgetFeature<AlertComponentSchema>;
export declare function useButtonWidget(): [FluentFormWidgetFeature<ButtonComponentSchema>, FluentFormWidgetFeature<ButtonGroupComponentSchema>];
export declare function useStepsWidget(): [FluentFormWidgetFeature<StepsComponentSchema>, FluentFormWidgetFeature<StepComponentSchema>];
export declare function useTabsWidget(): [FluentFormWidgetFeature<TabsComponentSchema>, FluentFormWidgetFeature<TabComponentSchema>];
export declare function useFormGroupWidget(): FluentFormWidgetFeature<FormGroupSchema>;
export declare function useFormArrayWidget(): FluentFormWidgetFeature<FormArraySchema>;
