import { AbstractControl, FormArray, FormControl, FormGroup } from '@angular/forms';
import { AnyArray, AnyObject } from '@ngify/types';
import { FormArraySchema, FormGroupSchema, SchemaKey } from '../schemas';
import { AnyControlContainerSchema, AnyControlSchema, AnySchema } from '../schemas/index.schema';
import * as i0 from "@angular/core";
/**
 * @internal
 */
export declare class FormUtil {
    private readonly schemaUtil;
    private readonly valueUtil;
    private readonly valueTransformer;
    createFormControl(schema: AnyControlSchema, model: AnyObject): FormControl;
    createFormGroup(schema: FormGroupSchema, model: AnyObject): FormGroup;
    createFormGroup(schemas: AnySchema[], model: AnyObject): FormGroup;
    createFormGroup(schemaOrSchemas: FormGroupSchema | AnySchema[], model: AnyObject): FormGroup;
    private createControlMap;
    createFormArray(schema: FormArraySchema, model: AnyArray): FormArray;
    createFormArrayElements(schemas: AnySchema[], model: AnyArray): AbstractControl<any, any>[];
    createAnyControl(schema: AnyControlSchema, model: AnyObject): FormControl;
    createAnyControl(schema: FormGroupSchema, model: AnyObject): FormGroup;
    createAnyControl(schema: FormArraySchema, model: AnyArray): FormArray;
    createAnyControl(schema: AnyControlSchema | AnyControlContainerSchema, model: AnyObject | AnyArray): AbstractControl;
    /**
     * 更新表单状态，目前包括更新 validator 与 enabled / disabled
     * @param form
     * @param model
     * @param schemas
     * @param completed
     */
    updateForm(form: FormGroup, model: AnyObject, schemas: AnySchema[], completed?: boolean): void;
    updateForm(form: FormArray, model: AnyArray, schemas: AnySchema[], completed?: boolean): void;
    /**
     * 从 form 赋值到 model
     * @param model
     * @param form
     * @param schemas
     */
    updateModel(model: AnyObject, form: FormGroup, schemas: AnySchema[]): AnyObject;
    updateModel(model: AnyArray, form: FormArray, schemas: AnySchema[]): AnyArray;
    static ɵfac: i0.ɵɵFactoryDeclaration<FormUtil, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<FormUtil>;
}
/**
 * 根据 form 的类型自动选择使用 .get() 还是 .at() 来获取子控件
 * @param form
 * @param key
 * @returns
 */
export declare function getChildControl(form: FormGroup | FormArray, key: SchemaKey): AbstractControl | null;
